package com.jiuqi.syntax;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

public class CommonData implements Cloneable {
   public static int CD_NEW_COUNT = 0;
   public static int CD_LIVE_COUNT = 0;
   public static final int NULL_TYPE = 0;
   public static final int INT_TYPE = 1;
   public static final int REAL_TYPE = 2;
   public static final int STRING_TYPE = 3;
   public static final int EXP_TYPE = 4;
   public static final int BOOL_TYPE = 5;
   public static final int DATE_TYPE = 6;
   public static final int MEMO_TYPE = 7;
   public static final int OBJECT_TYPE = 8;
   public static final int DECIMAL_TYPE = 10;
   public int type = 0;
   public boolean isNull = false;
   public int cdInt = 0;
   public double cdReal = 0.0;
   public String cdString = "";
   public boolean cdBool = false;
   public Calendar cdDate = null;
   public Object cdObject = null;
   public BigDecimal cdDecimal = null;
   protected static final DataOperator[] operators = new DataOperator[16];

   static {
      operators[0] = new DataOperator() {
         public int compare(CommonData v1, CommonData v2, double delta) {
            return 0;
         }
      };
      operators[1] = new DataOperator() {
         public int compare(CommonData v1, CommonData v2, double delta) {
            return v1.cdInt - v2.cdInt;
         }
      };
      operators[2] = new DataOperator() {
         public int compare(CommonData v1, CommonData v2, double delta) {
            if (delta == 0.0) {
               if (Math.abs(v1.cdReal - v2.cdReal) < 1.0E-5) {
                  return 0;
               } else {
                  return v1.cdReal > v2.cdReal ? 1 : -1;
               }
            } else if (Math.abs(v1.cdReal - v2.cdReal) < delta) {
               return 0;
            } else {
               return v1.cdReal > v2.cdReal ? 1 : -1;
            }
         }
      };
      operators[3] = new DataOperator() {
         public int compare(CommonData v1, CommonData v2, double delta) {
            return v1.cdString.compareTo(v2.cdString);
         }
      };
      operators[4] = operators[3];
      operators[5] = new DataOperator() {
         public int compare(CommonData v1, CommonData v2, double delta) {
            return v1.cdBool == v2.cdBool ? 0 : 255;
         }
      };
      operators[6] = new DataOperator() {
         public int compare(CommonData v1, CommonData v2, double delta) {
            if (v1.cdDate != null && v2.cdDate != null) {
               if (v1.cdDate.before(v2.cdDate)) {
                  return -1;
               } else {
                  return v1.cdDate.after(v2.cdDate) ? 1 : 0;
               }
            } else {
               return 255;
            }
         }
      };
      operators[10] = new DataOperator() {
         public int compare(CommonData v1, CommonData v2, double delta) {
            if (v1.cdDecimal != null && v2.cdDecimal != null) {
               if (delta == 0.0) {
                  delta = 1.0E-5;
               }

               BigDecimal comResult = v1.cdDecimal.subtract(v2.cdDecimal);
               if (comResult.abs().compareTo(new BigDecimal(delta)) < 0) {
                  return 0;
               } else {
                  return comResult.signum() > 0 ? 1 : -1;
               }
            } else {
               return 255;
            }
         }
      };
   }

   public double getIntRealValue() {
      if (this.type == 1) {
         return (double)this.cdInt;
      } else {
         return this.type == 2 ? this.cdReal : 0.0;
      }
   }

   public CommonData() {
      ++CD_NEW_COUNT;
      ++CD_LIVE_COUNT;
   }

   public Object clone() {
      CommonData d = new CommonData();
      this.assignTo(d);
      return d;
   }

   public void assign(CommonData source) {
      this.type = source.type;
      this.isNull = source.isNull;
      this.initNullValue();
      if (!this.isNull) {
         switch (this.type) {
            case 0:
            case 9:
            default:
               break;
            case 1:
               this.cdInt = source.cdInt;
               break;
            case 2:
               this.cdReal = source.cdReal;
               break;
            case 3:
               this.cdString = source.cdString;
               break;
            case 4:
               this.cdString = source.cdString;
               break;
            case 5:
               this.cdBool = source.cdBool;
               break;
            case 6:
               this.cdDate = source.cdDate;
               break;
            case 7:
               this.cdObject = source.cdObject;
            case 8:
               this.cdObject = source.cdObject;
               break;
            case 10:
               this.cdDecimal = source.cdDecimal;
         }
      }

   }

   public void assignTo(CommonData dest) {
      dest.assign(this);
   }

   public void initValue() {
      switch (this.type) {
         case 0:
         case 9:
         default:
            break;
         case 1:
            this.cdInt = 0;
            break;
         case 2:
            this.cdReal = 0.0;
            break;
         case 3:
            this.cdString = "";
            break;
         case 4:
            this.cdString = "";
            break;
         case 5:
            this.cdBool = false;
            break;
         case 6:
            this.cdDate = null;
            break;
         case 7:
            this.cdObject = null;
            break;
         case 8:
            this.cdObject = null;
            break;
         case 10:
            this.cdDecimal = new BigDecimal(0);
      }

   }

   public void initNullValue() {
      if (this.isNull) {
         this.initValue();
      }

   }

   public int compareTo(CommonData other) {
      return this.compareTo(other, 0.0);
   }

   public final int compareTo(CommonData other, double diffValue) {
      if (other == null) {
         return 255;
      } else if (this.type != other.type) {
         return 255;
      } else if (this.isNull && other.isNull) {
         return 0;
      } else {
         if (this.isNull != other.isNull) {
            this.initNullValue();
            other.initNullValue();
         }

         return operators[this.type] == null ? 255 : operators[this.type].compare(this, other, diffValue);
      }
   }

   public boolean equals(CommonData other) {
      return this.compareTo(other, 0.0) == 0;
   }

   public boolean equals(CommonData other, double diffValue) {
      return this.compareTo(other, diffValue) == 0;
   }

   public String toString() {
      String s = new String();
      switch (this.type) {
         case 0:
            s = "";
            break;
         case 1:
            s = String.valueOf(this.cdInt);
            break;
         case 2:
            s = String.valueOf(this.cdReal);
            break;
         case 3:
            s = this.cdString;
            break;
         case 4:
            s = this.cdString;
            break;
         case 5:
            s = String.valueOf(this.cdBool);
            break;
         case 6:
            if (this.cdDate == null) {
               s = "";
            } else {
               s = this.cdDate.toString();
            }
            break;
         case 7:
            s = "";
            break;
         case 8:
            s = "";
         case 9:
         default:
            break;
         case 10:
            if (this.cdDecimal == null) {
               s = "";
            } else {
               s = this.cdDecimal.toString();
            }
      }

      return s;
   }

   public void setDateValue(Date dateValue) {
      this.type = 6;
      this.isNull = dateValue == null;
      if (!this.isNull && this.cdDate == null) {
         this.cdDate = new GregorianCalendar();
      }

      if (dateValue != null) {
         this.cdDate.setTime(dateValue);
      } else if (this.cdDate != null) {
         this.cdDate.clear();
      }

   }

   public Calendar getValidcdDate() {
      if (this.cdDate == null) {
         this.cdDate = Calendar.getInstance();
         this.cdDate.getTime().setTime(0L);
         this.cdDate.set(1, 1899);
         this.cdDate.set(2, 1);
         this.cdDate.set(5, 1);
      }

      return this.cdDate;
   }

   protected void finalize() throws Throwable {
      --CD_LIVE_COUNT;
      super.finalize();
   }

   protected abstract static class DataOperator {
      public abstract int compare(CommonData var1, CommonData var2, double var3);
   }
}
